@@ -656,6 +656,11 @@ gsd_idle_monitor_get_idletime (GsdIdleMonitor *monitor)
 {
   XSyncValue value;
 
+  g_return_val_if_fail (GSD_IS_IDLE_MONITOR (monitor), -1);
+
+  if (monitor->counter == None)
+    return -1;
+
   if (!XSyncQueryCounter (monitor->display, monitor->counter, &value))
     return -1;
 
@@ -945,11 +950,6 @@ init_xsync_global (void)
         xsync->have_xsync = TRUE;
         XSyncSetPriority (xsync->display, None, 10);
       }
-
-    g_warning ("Attempted to init Xsync, found version %d.%d error base %d event base %d\n",
-                  major, minor,
-                  xsync->sync_error_base,
-                  xsync->sync_event_base);
 }
 
 void
