@@ -107,7 +107,7 @@ render_icon (GIcon *icon, gint icon_size)
 
 		if (info) {
 			pixbuf = gtk_icon_info_load_icon (info, NULL);
-		        gtk_icon_info_free (info);
+		        g_object_unref (info);
 		}
 
 		if (pixbuf == NULL) {
@@ -734,6 +734,7 @@ show_dialog:
 	gtk_label_set_markup (GTK_LABEL (label), markup);
 	g_free (markup);
 	gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
+	gtk_label_set_max_width_chars (GTK_LABEL (label), 72);
 	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
 	gtk_box_pack_start (GTK_BOX (vbox), label, TRUE, TRUE, 0);
 
@@ -744,6 +745,7 @@ show_dialog:
 	gtk_label_set_markup (GTK_LABEL (label), markup);
 	g_free (markup);
 	gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
+	gtk_label_set_max_width_chars (GTK_LABEL (label), 72);
 	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
 	gtk_box_pack_start (GTK_BOX (vbox), label, TRUE, TRUE, 0);
 
