@@ -38,11 +38,10 @@
 #include <gdk/gdk.h>
 #include <gdk/gdkx.h>
 
-#define GNOME_DESKTOP_USE_UNSTABLE_API
-#include <libgnome-desktop/gnome-bg.h>
 #include <X11/Xatom.h>
 
-#include "gnome-settings-session.h"
+#include "gsd-bg.h"
+#include "gnome-settings-bus.h"
 #include "gnome-settings-profile.h"
 #include "gsd-background-manager.h"
 
@@ -51,9 +50,10 @@
 struct GsdBackgroundManagerPrivate
 {
         GSettings   *settings;
-        GnomeBG     *bg;
+        GSettings   *usettings;
+        GsdBG     *bg;
 
-        GnomeBGCrossfade *fade;
+        GsdBGCrossfade *fade;
 
         GDBusProxy  *proxy;
         guint        proxy_signal_id;
@@ -73,7 +73,7 @@ static gpointer manager_object = NULL;
 static gboolean
 dont_draw_background (GsdBackgroundManager *manager)
 {
-        return !g_settings_get_boolean (manager->priv->settings,
+        return !g_settings_get_boolean (manager->priv->usettings,
                                         "draw-background");
 }
 
@@ -209,7 +209,7 @@ draw_background (GsdBackgroundManager *manager,
 
                 root_window = gdk_screen_get_root_window (screen);
 
-                surface = gnome_bg_create_surface (manager->priv->bg,
+                surface = gsd_bg_create_surface (manager->priv->bg,
                                                    root_window,
                                                    gdk_screen_get_width (screen),
                                                    gdk_screen_get_height (screen),
@@ -221,12 +221,12 @@ draw_background (GsdBackgroundManager *manager,
                                 g_object_unref (manager->priv->fade);
                         }
 
-                        manager->priv->fade = gnome_bg_set_surface_as_root_with_crossfade (screen, surface);
+                        manager->priv->fade = gsd_bg_set_surface_as_root_with_crossfade (screen, surface);
                         g_signal_connect_swapped (manager->priv->fade, "finished",
                                                   G_CALLBACK (on_crossfade_finished),
                                                   manager);
                 } else {
-                        gnome_bg_set_surface_as_root (screen, surface);
+                        gsd_bg_set_surface_as_root (screen, surface);
                 }
 
                 cairo_surface_destroy (surface);
@@ -236,7 +236,7 @@ draw_background (GsdBackgroundManager *manager,
 }
 
 static void
-on_bg_transitioned (GnomeBG              *bg,
+on_bg_transitioned (GsdBG              *bg,
                     GsdBackgroundManager *manager)
 {
         draw_background (manager, FALSE);
@@ -248,7 +248,7 @@ settings_change_event_cb (GSettings            *settings,
                           gint                  n_keys,
                           GsdBackgroundManager *manager)
 {
-        gnome_bg_load_from_preferences (manager->priv->bg,
+        gsd_bg_load_from_preferences (manager->priv->bg,
                                         manager->priv->settings);
         return FALSE;
 }
@@ -270,7 +270,7 @@ watch_bg_preferences (GsdBackgroundManager *manager)
 }
 
 static void
-on_bg_changed (GnomeBG              *bg,
+on_bg_changed (GsdBG              *bg,
                GsdBackgroundManager *manager)
 {
         draw_background (manager, TRUE);
@@ -281,7 +281,7 @@ setup_bg (GsdBackgroundManager *manager)
 {
         g_return_if_fail (manager->priv->bg == NULL);
 
-        manager->priv->bg = gnome_bg_new ();
+        manager->priv->bg = gsd_bg_new ();
 
         g_signal_connect (manager->priv->bg,
                           "changed",
@@ -295,7 +295,7 @@ setup_bg (GsdBackgroundManager *manager)
 
         connect_screen_signals (manager);
         watch_bg_preferences (manager);
-        gnome_bg_load_from_preferences (manager->priv->bg,
+        gsd_bg_load_from_preferences (manager->priv->bg,
                                         manager->priv->settings);
 }
 
@@ -335,7 +335,7 @@ static void
 draw_background_after_session_loads (GsdBackgroundManager *manager)
 {
         manager->priv->proxy =
-                gnome_settings_session_get_session_proxy ();
+                G_DBUS_PROXY (gnome_settings_bus_get_session_proxy ());
 
         manager->priv->proxy_signal_id = g_signal_connect (manager->priv->proxy,
                                                            "g-signal",
@@ -399,80 +399,80 @@ draw_background_changed (GSettings            *settings,
 static void
 set_accountsservice_background (const gchar *background)
 {
-  GDBusProxy *proxy = NULL;
-  GDBusProxy *user = NULL;
-  GVariant *variant = NULL;
+  GDBusConnection *bus;
+  GVariant *variant;
   GError *error = NULL;
   gchar *object_path = NULL;
 
-  proxy = g_dbus_proxy_new_for_bus_sync (G_BUS_TYPE_SYSTEM,
-                                         G_DBUS_PROXY_FLAGS_NONE,
-                                         NULL,
-                                         "org.freedesktop.Accounts",
-                                         "/org/freedesktop/Accounts",
-                                         "org.freedesktop.Accounts",
-                                         NULL,
-                                         &error);
-
-  if (proxy == NULL) {
-    g_warning ("Failed to contact accounts service: %s", error->message);
+  bus = g_bus_get_sync (G_BUS_TYPE_SYSTEM, NULL, &error);
+  if (bus == NULL) {
+    g_warning ("Failed to get system bus: %s", error->message);
     g_error_free (error);
     return;
   }
 
-  variant = g_dbus_proxy_call_sync (proxy,
-                                    "FindUserByName",
-                                    g_variant_new ("(s)", g_get_user_name ()),
-                                    G_DBUS_CALL_FLAGS_NONE,
-                                    -1,
-                                    NULL,
-                                    &error);
-
+  variant = g_dbus_connection_call_sync (bus,
+                                         "org.freedesktop.Accounts",
+                                         "/org/freedesktop/Accounts",
+                                         "org.freedesktop.Accounts",
+                                         "FindUserByName",
+                                         g_variant_new ("(s)", g_get_user_name ()),
+                                         G_VARIANT_TYPE ("(o)"),
+                                         G_DBUS_CALL_FLAGS_NONE,
+                                         -1,
+                                         NULL,
+                                         &error);
   if (variant == NULL) {
     g_warning ("Could not contact accounts service to look up '%s': %s",
                g_get_user_name (), error->message);
     g_error_free (error);
-    goto bail;
+    g_object_unref (bus);
+    return;
   }
-
   g_variant_get (variant, "(o)", &object_path);
-  user = g_dbus_proxy_new_for_bus_sync (G_BUS_TYPE_SYSTEM,
-                                        G_DBUS_PROXY_FLAGS_NONE,
-                                        NULL,
-                                        "org.freedesktop.Accounts",
-                                        object_path,
-                                        "org.freedesktop.Accounts.User",
-                                        NULL,
-                                        &error);
-  g_free (object_path);
-
-  if (user == NULL) {
-    g_warning ("Could not create proxy for user '%s': %s",
-               g_variant_get_string (variant, NULL), error->message);
-    g_error_free (error);
-    goto bail;
-  }
   g_variant_unref (variant);
 
-  variant = g_dbus_proxy_call_sync (user,
-                                    "SetBackgroundFile",
-                                    g_variant_new ("(s)", background ? background : ""),
-                                    G_DBUS_CALL_FLAGS_NONE,
-                                    -1,
-                                    NULL,
-                                    &error);
-
-  if (variant == NULL) {
-    g_warning ("Failed to set the background '%s': %s", background, error->message);
-    g_error_free (error);
-    goto bail;
-  }
-
-bail:
-  if (proxy != NULL)
-    g_object_unref (proxy);
+  variant = g_dbus_connection_call_sync (bus,
+                                         "org.freedesktop.Accounts",
+                                         object_path,
+                                         "org.freedesktop.DBus.Properties",
+                                         "Set",
+                                         g_variant_new ("(ssv)",
+                                                        "org.freedesktop.DisplayManager.AccountsService",
+                                                        "BackgroundFile",
+                                                        g_variant_new_string (background ? background : "")),
+                                         G_VARIANT_TYPE ("()"),
+                                         G_DBUS_CALL_FLAGS_NONE,
+                                         -1,
+                                         NULL,
+                                         &error);
   if (variant != NULL)
     g_variant_unref (variant);
+  else {
+    g_warning ("Failed to set the background '%s': %s", background, error->message);
+    g_clear_error (&error);
+  }
+
+  /* Also attempt the old method (patch not upstreamed into AccountsService */
+  variant = g_dbus_connection_call_sync (bus,
+                                         "org.freedesktop.Accounts",
+                                         object_path,
+                                         "org.freedesktop.Accounts.User",
+                                         "SetBackgroundFile",
+                                         g_variant_new ("(s)", background ? background : ""),
+                                         G_VARIANT_TYPE ("()"),
+                                         G_DBUS_CALL_FLAGS_NONE,
+                                         -1,
+                                         NULL,
+                                         &error);
+  if (variant != NULL)
+    g_variant_unref (variant);
+  else {
+    g_warning ("Failed to set the background '%s': %s", background, error->message);
+    g_clear_error (&error);
+  }
+
+  g_object_unref (bus);
 }
 
 static void
@@ -498,8 +498,11 @@ gsd_background_manager_start (GsdBackgroundManager *manager,
         gnome_settings_profile_start (NULL);
 
         manager->priv->settings = g_settings_new ("org.gnome.desktop.background");
-        g_signal_connect (manager->priv->settings, "changed::draw-background",
+        manager->priv->usettings = g_settings_new ("com.canonical.unity.desktop.background");
+
+        g_signal_connect (manager->priv->usettings, "changed::draw-background",
                           G_CALLBACK (draw_background_changed), manager);
+
         g_signal_connect (manager->priv->settings, "changed::picture-uri",
                           G_CALLBACK (picture_uri_changed), manager);
 
@@ -542,11 +545,20 @@ gsd_background_manager_stop (GsdBackgroundManager *manager)
                                               settings_change_event_cb,
                                               manager);
 
+        g_signal_handlers_disconnect_by_func (manager->priv->usettings,
+                                              settings_change_event_cb,
+                                              manager);
+
         if (p->settings != NULL) {
                 g_object_unref (p->settings);
                 p->settings = NULL;
         }
 
+        if (p->usettings != NULL) {
+                g_object_unref (p->usettings);
+                p->usettings = NULL;
+        }
+
         if (p->bg != NULL) {
                 g_object_unref (p->bg);
                 p->bg = NULL;
