@@ -27,8 +27,6 @@
 #include <gdk/gdk.h>
 #include <gdk/gdkx.h>
 #include <X11/Xatom.h>
-#define GNOME_DESKTOP_USE_UNSTABLE_API
-#include <libgnome-desktop/gnome-rr.h>
 
 #include <libwacom/libwacom.h>
 #include <X11/extensions/XInput.h>
@@ -38,6 +36,7 @@
 
 #include "gsd-enums.h"
 #include "gsd-wacom-device.h"
+#include "gsd-rr.h"
 
 #define GSD_WACOM_STYLUS_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GSD_TYPE_WACOM_STYLUS, GsdWacomStylusPrivate))
 
@@ -48,14 +47,14 @@
 #define WACOM_BUTTON_SCHEMA "org.gnome.settings-daemon.peripherals.wacom.tablet-button"
 
 static struct {
-	GnomeRRRotation  rotation;
+	GsdRRRotation  rotation;
 	GsdWacomRotation rotation_wacom;
 	const gchar     *rotation_string;
 } rotation_table[] = {
-	{ GNOME_RR_ROTATION_0,   GSD_WACOM_ROTATION_NONE, "none" },
-	{ GNOME_RR_ROTATION_90,  GSD_WACOM_ROTATION_CCW,  "ccw"  },
-	{ GNOME_RR_ROTATION_180, GSD_WACOM_ROTATION_HALF, "half" },
-	{ GNOME_RR_ROTATION_270, GSD_WACOM_ROTATION_CW,   "cw"   }
+	{ GSD_RR_ROTATION_0,   GSD_WACOM_ROTATION_NONE, "none" },
+	{ GSD_RR_ROTATION_90,  GSD_WACOM_ROTATION_CCW,  "ccw"  },
+	{ GSD_RR_ROTATION_180, GSD_WACOM_ROTATION_HALF, "half" },
+	{ GSD_RR_ROTATION_270, GSD_WACOM_ROTATION_CW,   "cw"   }
 };
 
 static WacomDeviceDatabase *db = NULL;
@@ -511,14 +510,14 @@ get_device_type (XDeviceInfo *dev)
 
 /* Finds an output which matches the given EDID information. Any NULL
  * parameter will be interpreted to match any value. */
-static GnomeRROutput *
-find_output_by_edid (GnomeRRScreen *rr_screen, const gchar *vendor, const gchar *product, const gchar *serial)
+static GsdRROutput *
+find_output_by_edid (GsdRRScreen *rr_screen, const gchar *vendor, const gchar *product, const gchar *serial)
 {
-	GnomeRROutput **rr_outputs;
-	GnomeRROutput *retval = NULL;
+	GsdRROutput **rr_outputs;
+	GsdRROutput *retval = NULL;
 	guint i;
 
-	rr_outputs = gnome_rr_screen_list_outputs (rr_screen);
+	rr_outputs = gsd_rr_screen_list_outputs (rr_screen);
 
 	for (i = 0; rr_outputs[i] != NULL; i++) {
 		gchar *o_vendor_s;
@@ -528,10 +527,10 @@ find_output_by_edid (GnomeRRScreen *rr_screen, const gchar *vendor, const gchar
 		int o_serial;
 		gboolean match;
 
-		if (!gnome_rr_output_is_connected (rr_outputs[i]))
+		if (!gsd_rr_output_is_connected (rr_outputs[i]))
 			continue;
 
-		if (!gnome_rr_output_get_ids_from_edid (rr_outputs[i],
+		if (!gsd_rr_output_get_ids_from_edid (rr_outputs[i],
 						        &o_vendor_s,
 						        &o_product,
 						        &o_serial))
@@ -564,19 +563,19 @@ find_output_by_edid (GnomeRRScreen *rr_screen, const gchar *vendor, const gchar
 	return retval;
 }
 
-static GnomeRROutput*
-find_builtin_output (GnomeRRScreen *rr_screen)
+static GsdRROutput*
+find_builtin_output (GsdRRScreen *rr_screen)
 {
-	GnomeRROutput **rr_outputs;
-	GnomeRROutput *retval = NULL;
+	GsdRROutput **rr_outputs;
+	GsdRROutput *retval = NULL;
 	guint i;
 
-	rr_outputs = gnome_rr_screen_list_outputs (rr_screen);
+	rr_outputs = gsd_rr_screen_list_outputs (rr_screen);
 	for (i = 0; rr_outputs[i] != NULL; i++) {
-		if (!gnome_rr_output_is_connected (rr_outputs[i]))
+		if (!gsd_rr_output_is_connected (rr_outputs[i]))
 			continue;
 
-		if (gnome_rr_output_is_laptop(rr_outputs[i])) {
+		if (gsd_rr_output_is_laptop(rr_outputs[i])) {
 			retval = rr_outputs[i];
 			break;
 		}
@@ -588,10 +587,10 @@ find_builtin_output (GnomeRRScreen *rr_screen)
 	return retval;
 }
 
-static GnomeRROutput *
-find_output_by_heuristic (GnomeRRScreen *rr_screen, GsdWacomDevice *device)
+static GsdRROutput *
+find_output_by_heuristic (GsdRRScreen *rr_screen, GsdWacomDevice *device)
 {
-	GnomeRROutput *rr_output;
+	GsdRROutput *rr_output;
 
 	/* TODO: This heuristic will fail for non-Wacom display
 	 * tablets and may give the wrong result if multiple Wacom
@@ -605,14 +604,14 @@ find_output_by_heuristic (GnomeRRScreen *rr_screen, GsdWacomDevice *device)
 	return rr_output;
 }
 
-static GnomeRROutput *
-find_output_by_display (GnomeRRScreen *rr_screen, GsdWacomDevice *device)
+static GsdRROutput *
+find_output_by_display (GsdRRScreen *rr_screen, GsdWacomDevice *device)
 {
 	gsize n;
 	GSettings *tablet;
 	GVariant *display;
 	const gchar **edid;
-	GnomeRROutput *ret;
+	GsdRROutput *ret;
 
 	if (device == NULL)
 		return NULL;
@@ -640,32 +639,32 @@ out:
 }
 
 static gboolean
-is_on (GnomeRROutput *output)
+is_on (GsdRROutput *output)
 {
-	GnomeRRCrtc *crtc;
+	GsdRRCrtc *crtc;
 
-	crtc = gnome_rr_output_get_crtc (output);
+	crtc = gsd_rr_output_get_crtc (output);
 	if (!crtc)
 		return FALSE;
-	return gnome_rr_crtc_get_current_mode (crtc) != NULL;
+	return gsd_rr_crtc_get_current_mode (crtc) != NULL;
 }
 
-static GnomeRROutput *
-find_output_by_monitor (GnomeRRScreen *rr_screen,
+static GsdRROutput *
+find_output_by_monitor (GsdRRScreen *rr_screen,
 			GdkScreen     *screen,
 			int            monitor)
 {
-	GnomeRROutput **rr_outputs;
-	GnomeRROutput *ret;
+	GsdRROutput **rr_outputs;
+	GsdRROutput *ret;
 	guint i;
 
 	ret = NULL;
 
-	rr_outputs = gnome_rr_screen_list_outputs (rr_screen);
+	rr_outputs = gsd_rr_screen_list_outputs (rr_screen);
 
 	for (i = 0; rr_outputs[i] != NULL; i++) {
-		GnomeRROutput *rr_output;
-		GnomeRRCrtc *crtc;
+		GsdRROutput *rr_output;
+		GsdRRCrtc *crtc;
 		int x, y;
 
 		rr_output = rr_outputs[i];
@@ -673,11 +672,11 @@ find_output_by_monitor (GnomeRRScreen *rr_screen,
 		if (!is_on (rr_output))
 			continue;
 
-		crtc = gnome_rr_output_get_crtc (rr_output);
+		crtc = gsd_rr_output_get_crtc (rr_output);
 		if (!crtc)
 			continue;
 
-		gnome_rr_crtc_get_position (crtc, &x, &y);
+		gsd_rr_crtc_get_position (crtc, &x, &y);
 
 		if (monitor == gdk_screen_get_monitor_at_point (screen, x, y)) {
 			ret = rr_output;
@@ -693,7 +692,7 @@ find_output_by_monitor (GnomeRRScreen *rr_screen,
 
 static void
 set_display_by_output (GsdWacomDevice  *device,
-                       GnomeRROutput   *rr_output)
+                       GsdRROutput   *rr_output)
 {
 	GSettings   *tablet;
 	GVariant    *c_array;
@@ -712,7 +711,7 @@ set_display_by_output (GsdWacomDevice  *device,
 	}
 
 	if (rr_output == NULL ||
-	    !gnome_rr_output_get_ids_from_edid (rr_output,
+	    !gsd_rr_output_get_ids_from_edid (rr_output,
 					        &o_vendor_s,
 					        &o_product,
 					        &o_serial)) {
@@ -736,7 +735,7 @@ set_display_by_output (GsdWacomDevice  *device,
 }
 
 static GsdWacomRotation
-get_rotation_wacom (GnomeRRRotation rotation)
+get_rotation_wacom (GsdRRRotation rotation)
 {
         guint i;
 
@@ -752,14 +751,14 @@ gsd_wacom_device_set_display (GsdWacomDevice *device,
                               int             monitor)
 {
 	GError *error = NULL;
-	GnomeRRScreen *rr_screen;
-	GnomeRROutput *output = NULL;
+	GsdRRScreen *rr_screen;
+	GsdRROutput *output = NULL;
 
         g_return_if_fail (GSD_IS_WACOM_DEVICE (device));
 
-	rr_screen = gnome_rr_screen_new (gdk_screen_get_default (), &error);
+	rr_screen = gsd_rr_screen_new (gdk_screen_get_default (), &error);
 	if (rr_screen == NULL) {
-		g_warning ("Failed to create GnomeRRScreen: %s", error->message);
+		g_warning ("Failed to create GsdRRScreen: %s", error->message);
 		g_error_free (error);
 		return;
 	}
@@ -771,11 +770,11 @@ gsd_wacom_device_set_display (GsdWacomDevice *device,
 	g_object_unref (rr_screen);
 }
 
-static GnomeRROutput *
-find_output (GnomeRRScreen  *rr_screen,
+static GsdRROutput *
+find_output (GsdRRScreen  *rr_screen,
 	     GsdWacomDevice *device)
 {
-	GnomeRROutput *rr_output;
+	GsdRROutput *rr_output;
 	rr_output = find_output_by_display (rr_screen, device);
 
 	if (rr_output == NULL) {
@@ -823,17 +822,17 @@ int
 gsd_wacom_device_get_display_monitor (GsdWacomDevice *device)
 {
 	GError *error = NULL;
-	GnomeRRScreen *rr_screen;
-	GnomeRROutput *rr_output;
-	GnomeRRMode *mode;
-	GnomeRRCrtc *crtc;
+	GsdRRScreen *rr_screen;
+	GsdRROutput *rr_output;
+	GsdRRMode *mode;
+	GsdRRCrtc *crtc;
 	gint area[4];
 
         g_return_val_if_fail (GSD_IS_WACOM_DEVICE (device), GSD_WACOM_SET_ALL_MONITORS);
 
-	rr_screen = gnome_rr_screen_new (gdk_screen_get_default (), &error);
+	rr_screen = gsd_rr_screen_new (gdk_screen_get_default (), &error);
 	if (rr_screen == NULL) {
-		g_warning ("Failed to create GnomeRRScreen: %s", error->message);
+		g_warning ("Failed to create GsdRRScreen: %s", error->message);
 		g_error_free (error);
 		return GSD_WACOM_SET_ALL_MONITORS;
 	}
@@ -850,12 +849,12 @@ gsd_wacom_device_get_display_monitor (GsdWacomDevice *device)
 		return GSD_WACOM_SET_ALL_MONITORS;
 	}
 
-	crtc = gnome_rr_output_get_crtc (rr_output);
-	gnome_rr_crtc_get_position (crtc, &area[0], &area[1]);
+	crtc = gsd_rr_output_get_crtc (rr_output);
+	gsd_rr_crtc_get_position (crtc, &area[0], &area[1]);
 
-	mode = gnome_rr_crtc_get_current_mode (crtc);
-	area[2] = gnome_rr_mode_get_width (mode);
-	area[3] = gnome_rr_mode_get_height (mode);
+	mode = gsd_rr_crtc_get_current_mode (crtc);
+	area[2] = gsd_rr_mode_get_width (mode);
+	area[3] = gsd_rr_mode_get_height (mode);
 
 	g_object_unref (rr_screen);
 
@@ -904,22 +903,22 @@ GsdWacomRotation
 gsd_wacom_device_get_display_rotation (GsdWacomDevice *device)
 {
 	GError *error = NULL;
-	GnomeRRScreen *rr_screen;
-	GnomeRROutput *rr_output;
-	GnomeRRRotation rotation = GNOME_RR_ROTATION_0;
+	GsdRRScreen *rr_screen;
+	GsdRROutput *rr_output;
+	GsdRRRotation rotation = GSD_RR_ROTATION_0;
 
-	rr_screen = gnome_rr_screen_new (gdk_screen_get_default (), &error);
+	rr_screen = gsd_rr_screen_new (gdk_screen_get_default (), &error);
 	if (rr_screen == NULL) {
-		g_warning ("Failed to create GnomeRRScreen: %s", error->message);
+		g_warning ("Failed to create GsdRRScreen: %s", error->message);
 		g_error_free (error);
 		return GSD_WACOM_ROTATION_NONE;
 	}
 
 	rr_output = find_output (rr_screen, device);
 	if (rr_output) {
-		GnomeRRCrtc *crtc = gnome_rr_output_get_crtc (rr_output);
+		GsdRRCrtc *crtc = gsd_rr_output_get_crtc (rr_output);
 		if (crtc)
-			rotation = gnome_rr_crtc_get_current_rotation (crtc);
+			rotation = gsd_rr_crtc_get_current_rotation (crtc);
 	}
 	g_object_unref (rr_screen);
 
