@@ -38,8 +38,6 @@
 #include <X11/extensions/XTest.h>
 #include <X11/keysym.h>
 #include <Xwacom.h>
-#define GNOME_DESKTOP_USE_UNSTABLE_API
-#include <libgnome-desktop/gnome-rr.h>
 
 #include "gsd-enums.h"
 #include "gsd-input-helper.h"
@@ -48,6 +46,7 @@
 #include "gsd-wacom-manager.h"
 #include "gsd-wacom-device.h"
 #include "gsd-wacom-osd-window.h"
+#include "gsd-rr.h"
 
 #define GSD_WACOM_MANAGER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GSD_TYPE_WACOM_MANAGER, GsdWacomManagerPrivate))
 
@@ -169,7 +168,7 @@ wacom_set_property (GsdWacomDevice *device,
 
 	xdev = open_device (device);
 	device_set_property (xdev, gsd_wacom_device_get_tool_name (device), property);
-	XCloseDevice (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), xdev);
+	xdevice_close (xdev);
 }
 
 static void
@@ -316,7 +315,7 @@ set_absolute (GsdWacomDevice  *device,
 	if (gdk_error_trap_pop ())
 		g_error ("Failed to set mode \"%s\" for \"%s\".",
 			 is_absolute ? "Absolute" : "Relative", gsd_wacom_device_get_tool_name (device));
-	XCloseDevice (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), xdev);
+	xdevice_close (xdev);
 }
 
 static void
@@ -463,7 +462,7 @@ set_device_buttonmap (GsdWacomDevice *device,
 
 	g_free (map);
 
-	XCloseDevice (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), xdev);
+	xdevice_close (xdev);
 }
 
 static void
@@ -689,7 +688,7 @@ reset_pad_buttons (GsdWacomDevice *device)
 	reset_touch_buttons (xdev, def_touchstrip_buttons, "Wacom Strip Buttons");
 	gdk_error_trap_pop_ignored ();
 
-	XCloseDevice (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), xdev);
+	xdevice_close (xdev);
 
 	/* Reset all the LEDs */
 	buttons = gsd_wacom_device_get_buttons (device);
@@ -1371,7 +1370,7 @@ gsd_wacom_manager_idle_cb (GsdWacomManager *manager)
  * position of the monitors attached to the screen change.
  */
 static void
-on_screen_changed_cb (GnomeRRScreen *rr_screen,
+on_screen_changed_cb (GsdRRScreen *rr_screen,
 		      GsdWacomManager *manager)
 {
 	GList *devices, *l;
@@ -1417,7 +1416,7 @@ init_screens (GsdWacomManager *manager)
         for (i = 0; i < gdk_display_get_n_screens (display); i++) {
                 GError *error = NULL;
                 GdkScreen *screen;
-                GnomeRRScreen *rr_screen;
+                GsdRRScreen *rr_screen;
 
                 screen = gdk_display_get_screen (display, i);
                 if (screen == NULL) {
@@ -1426,12 +1425,12 @@ init_screens (GsdWacomManager *manager)
                 manager->priv->screens = g_slist_append (manager->priv->screens, screen);
 
 		/*
-		 * We also keep a list of GnomeRRScreen to monitor changes such as rotation
+		 * We also keep a list of GsdRRScreen to monitor changes such as rotation
 		 * which are not reported by Gdk's "monitors-changed" callback
 		 */
-		rr_screen = gnome_rr_screen_new (screen, &error);
+		rr_screen = gsd_rr_screen_new (screen, &error);
 		if (rr_screen == NULL) {
-			g_warning ("Failed to create GnomeRRScreen: %s", error->message);
+			g_warning ("Failed to create GsdRRScreen: %s", error->message);
 			g_error_free (error);
 			continue;
 		}
@@ -1486,14 +1485,19 @@ gsd_wacom_manager_stop (GsdWacomManager *manager)
                 devices = gdk_device_manager_list_devices (p->device_manager, GDK_DEVICE_TYPE_SLAVE);
                 for (l = devices; l != NULL; l = l->next) {
                         GsdWacomDeviceType type;
+                        GsdWacomDevice *device;
+                        int id;
 
-                        type = gsd_wacom_device_get_device_type (l->data);
-                        if (type == WACOM_TYPE_PAD) {
-                                int id;
+                        id = gdk_x11_device_get_id (l->data);
+                        device = device_id_to_device (manager, id);
 
-                                id = get_device_id (l->data);
+                        if (!device)
+                                continue;
+
+                        type = gsd_wacom_device_get_device_type (device);
+
+                        if (type == WACOM_TYPE_PAD)
                                 grab_button (id, FALSE, manager->priv->screens);
-                        }
                 }
                 g_list_free (devices);
 
