@@ -44,6 +44,7 @@
 #include "gsd-xsettings-gtk.h"
 #include "xsettings-manager.h"
 #include "fontconfig-monitor.h"
+#include "wm-button-layout-translation.h"
 
 #define GNOME_XSETTINGS_MANAGER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GNOME_TYPE_XSETTINGS_MANAGER, GnomeXSettingsManagerPrivate))
 
@@ -51,6 +52,7 @@
 #define INTERFACE_SETTINGS_SCHEMA "org.gnome.desktop.interface"
 #define SOUND_SETTINGS_SCHEMA     "org.gnome.desktop.sound"
 #define PRIVACY_SETTINGS_SCHEMA     "org.gnome.desktop.privacy"
+#define WM_SETTINGS_SCHEMA        "org.gnome.desktop.wm.preferences"
 
 #define XSETTINGS_PLUGIN_SCHEMA "org.gnome.settings-daemon.plugins.xsettings"
 #define XSETTINGS_OVERRIDE_KEY  "overrides"
@@ -61,6 +63,7 @@
 #define TEXT_SCALING_FACTOR_KEY "text-scaling-factor"
 #define SCALING_FACTOR_KEY "scaling-factor"
 #define CURSOR_SIZE_KEY "cursor-size"
+#define CURSOR_THEME_KEY "cursor-theme"
 
 #define FONT_ANTIALIASING_KEY "antialiasing"
 #define FONT_HINTING_KEY      "hinting"
@@ -337,6 +340,24 @@ translate_string_string_toolbar (GnomeXSettingsManager *manager,
         }
 }
 
+static void
+translate_button_layout (GnomeXSettingsManager *manager,
+                         TranslationEntry      *trans,
+                         GVariant              *value)
+{
+        char *layout = g_variant_dup_string (value, NULL);
+        int i;
+
+        translate_wm_button_layout_to_gtk (layout);
+
+        for (i = 0; manager->priv->managers [i]; i++)
+                xsettings_manager_set_string (manager->priv->managers [i],
+                                              trans->xsetting_name,
+                                              layout);
+
+        g_free (layout);
+}
+
 static TranslationEntry translations [] = {
         { "org.gnome.settings-daemon.peripherals.mouse", "double-click",   "Net/DoubleClickTime",  translate_int_int },
         { "org.gnome.settings-daemon.peripherals.mouse", "drag-threshold", "Net/DndDragThreshold", translate_int_int },
@@ -373,7 +394,8 @@ static TranslationEntry translations [] = {
         { "org.gnome.desktop.sound", "input-feedback-sounds",      "Net/EnableInputFeedbackSounds", translate_bool_int },
 
         { "org.gnome.desktop.privacy", "recent-files-max-age",      "Gtk/RecentFilesMaxAge", translate_int_int },
-        { "org.gnome.desktop.privacy", "remember-recent-files",    "Gtk/RecentFilesEnabled", translate_bool_int }
+        { "org.gnome.desktop.privacy", "remember-recent-files",    "Gtk/RecentFilesEnabled", translate_bool_int },
+        { "org.gnome.desktop.wm.preferences", "button-layout",     "Gtk/DecorationLayout", translate_button_layout }
 };
 
 static gboolean
@@ -415,7 +437,7 @@ get_dpi_from_gsettings (GnomeXSettingsManager *manager)
 static int
 get_window_scale (GnomeXSettingsManager *manager)
 {
-	GSettings  *interface_settings;
+        GSettings  *interface_settings;
         int window_scale;
         GdkRectangle rect;
         GdkDisplay *display;
@@ -424,13 +446,43 @@ get_window_scale (GnomeXSettingsManager *manager)
         int monitor_scale;
         double dpi_x, dpi_y;
 
-	interface_settings = g_hash_table_lookup (manager->priv->settings, INTERFACE_SETTINGS_SCHEMA);
+        interface_settings = g_hash_table_lookup (manager->priv->settings, INTERFACE_SETTINGS_SCHEMA);
         window_scale =
                 g_settings_get_uint (interface_settings, SCALING_FACTOR_KEY);
         if (window_scale == 0) {
                 window_scale = 1;
+
+                /* Under Unity let the shell handle the scaling */
+                if (g_strcmp0 (g_getenv ("XDG_CURRENT_DESKTOP"), "Unity") == 0)
+                        goto out;
+
+                display = gdk_display_get_default ();
+                screen = gdk_display_get_default_screen (display);
+                gdk_screen_get_monitor_geometry (screen, 0, &rect);
+                width_mm = gdk_screen_get_monitor_width_mm (screen, 0);
+                height_mm = gdk_screen_get_monitor_height_mm (screen, 0);
+                monitor_scale = gdk_screen_get_monitor_scale_factor (screen, 0);
+
+                /* Somebody encoded the aspect ratio (16/9 or 16/10)
+                 * instead of the physical size */
+                if ((width_mm == 160 && height_mm == 90) ||
+                    (width_mm == 160 && height_mm == 100) ||
+                    (width_mm == 16 && height_mm == 9) ||
+                    (width_mm == 16 && height_mm == 10))
+                        goto out;
+
+                if (width_mm > 0 && height_mm > 0) {
+                        dpi_x = (double)rect.width * monitor_scale / (width_mm / 25.4);
+                        dpi_y = (double)rect.height * monitor_scale / (height_mm / 25.4);
+                        /* We don't completely trust these values so both
+                           must be high, and never pick higher ratio than
+                           2 automatically */
+                        if (dpi_x > HIDPI_LIMIT && dpi_y > HIDPI_LIMIT)
+                                window_scale = 2;
+                }
         }
 
+out:
         return window_scale;
 }
 
@@ -441,6 +493,7 @@ typedef struct {
         int         dpi;
         int         window_scale;
         int         cursor_size;
+        gchar      *cursor_theme;
         const char *rgba;
         const char *hintstyle;
 } GnomeXftSettings;
@@ -472,6 +525,7 @@ xft_settings_get (GnomeXSettingsManager *manager,
         settings->scaled_dpi = dpi * settings->window_scale * 1024;
         cursor_size = g_settings_get_int (interface_settings, CURSOR_SIZE_KEY);
         settings->cursor_size = cursor_size * settings->window_scale;
+        settings->cursor_theme = g_settings_get_string (interface_settings, CURSOR_THEME_KEY);
         settings->rgba = "rgb";
         settings->hintstyle = "hintfull";
 
@@ -525,6 +579,12 @@ xft_settings_get (GnomeXSettingsManager *manager,
         }
 }
 
+static void
+xft_settings_clear (GnomeXftSettings *settings)
+{
+        g_free (settings->cursor_theme);
+}
+
 static void
 xft_settings_set_xsettings (GnomeXSettingsManager *manager,
                             GnomeXftSettings      *settings)
@@ -542,6 +602,7 @@ xft_settings_set_xsettings (GnomeXSettingsManager *manager,
                 xsettings_manager_set_int (manager->priv->managers [i], "Xft/DPI", settings->scaled_dpi);
                 xsettings_manager_set_string (manager->priv->managers [i], "Xft/RGBA", settings->rgba);
                 xsettings_manager_set_int (manager->priv->managers [i], "Gtk/CursorThemeSize", settings->cursor_size);
+                xsettings_manager_set_string (manager->priv->managers [i], "Gtk/CursorThemeName", settings->cursor_theme);
         }
         gnome_settings_profile_end (NULL);
 }
@@ -603,6 +664,10 @@ xft_settings_set_xresources (GnomeXftSettings *settings)
                                 settings->hintstyle);
         update_property (add_string, "Xft.rgba",
                                 settings->rgba);
+        update_property (add_string, "Xcursor.size",
+                                g_ascii_dtostr (dpibuf, sizeof (dpibuf), (double) settings->cursor_size));
+        update_property (add_string, "Xcursor.theme",
+                                settings->cursor_theme);
 
         g_debug("xft_settings_set_xresources: new res '%s'", add_string->str);
 
@@ -629,6 +694,7 @@ update_xft_settings (GnomeXSettingsManager *manager)
         xft_settings_get (manager, &settings);
         xft_settings_set_xsettings (manager, &settings);
         xft_settings_set_xresources (&settings);
+        xft_settings_clear (&settings);
 
         gnome_settings_profile_end (NULL);
 }
@@ -828,7 +894,7 @@ find_translation_entry (GSettings *settings, const char *key)
         guint i;
         char *schema;
 
-        g_object_get (settings, "schema", &schema, NULL);
+        g_object_get (settings, "schema-id", &schema, NULL);
 
         for (i = 0; i < G_N_ELEMENTS (translations); i++) {
                 if (g_str_equal (schema, translations[i].gsettings_schema) &&
@@ -853,7 +919,9 @@ xsettings_callback (GSettings             *settings,
         GVariant         *value;
 
         if (g_str_equal (key, TEXT_SCALING_FACTOR_KEY) ||
-            g_str_equal (key, SCALING_FACTOR_KEY)) {
+            g_str_equal (key, SCALING_FACTOR_KEY) ||
+            g_str_equal (key, CURSOR_SIZE_KEY) ||
+            g_str_equal (key, CURSOR_THEME_KEY)) {
         	xft_callback (NULL, key, manager);
         	return;
 	}
@@ -951,7 +1019,7 @@ start_unity_monitor (GnomeXSettingsManager *manager)
 {
         notify_have_shell (manager);
         manager->priv->have_unity = TRUE;
-        manager->priv->shell_name_watch_id = g_bus_watch_name (G_BUS_TYPE_SESSION,
+        manager->priv->unity_name_watch_id = g_bus_watch_name (G_BUS_TYPE_SESSION,
                                                                "com.canonical.AppMenu.Registrar",
                                                                0,
                                                                on_unity_appeared,
@@ -989,6 +1057,14 @@ gnome_xsettings_manager_start (GnomeXSettingsManager *manager,
                              SOUND_SETTINGS_SCHEMA, g_settings_new (SOUND_SETTINGS_SCHEMA));
         g_hash_table_insert (manager->priv->settings,
                              PRIVACY_SETTINGS_SCHEMA, g_settings_new (PRIVACY_SETTINGS_SCHEMA));
+        g_hash_table_insert (manager->priv->settings,
+                             WM_SETTINGS_SCHEMA, g_settings_new (WM_SETTINGS_SCHEMA));
+
+        list = g_hash_table_get_values (manager->priv->settings);
+        for (l = list; l != NULL; l = l->next) {
+                g_signal_connect_object (G_OBJECT (l->data), "changed", G_CALLBACK (xsettings_callback), manager, 0);
+        }
+        g_list_free (list);
 
         for (i = 0; i < G_N_ELEMENTS (translations); i++) {
                 GVariant *val;
@@ -1007,12 +1083,6 @@ gnome_xsettings_manager_start (GnomeXSettingsManager *manager,
                 g_variant_unref (val);
         }
 
-        list = g_hash_table_get_values (manager->priv->settings);
-        for (l = list; l != NULL; l = l->next) {
-                g_signal_connect_object (G_OBJECT (l->data), "changed", G_CALLBACK (xsettings_callback), manager, 0);
-        }
-        g_list_free (list);
-
         /* Plugin settings (GTK modules and Xft) */
         manager->priv->plugin_settings = g_settings_new (XSETTINGS_PLUGIN_SCHEMA);
         g_signal_connect_object (manager->priv->plugin_settings, "changed", G_CALLBACK (plugin_callback), manager, 0);
@@ -1068,6 +1138,7 @@ gnome_xsettings_manager_stop (GnomeXSettingsManager *manager)
         }
 
         if (p->plugin_settings != NULL) {
+                g_signal_handlers_disconnect_by_data (p->plugin_settings, manager);
                 g_object_unref (p->plugin_settings);
                 p->plugin_settings = NULL;
         }
@@ -1088,26 +1159,11 @@ gnome_xsettings_manager_stop (GnomeXSettingsManager *manager)
         }
 }
 
-static GObject *
-gnome_xsettings_manager_constructor (GType                  type,
-                                     guint                  n_construct_properties,
-                                     GObjectConstructParam *construct_properties)
-{
-        GnomeXSettingsManager      *xsettings_manager;
-
-        xsettings_manager = GNOME_XSETTINGS_MANAGER (G_OBJECT_CLASS (gnome_xsettings_manager_parent_class)->constructor (type,
-                                                                                                                  n_construct_properties,
-                                                                                                                  construct_properties));
-
-        return G_OBJECT (xsettings_manager);
-}
-
 static void
 gnome_xsettings_manager_class_init (GnomeXSettingsManagerClass *klass)
 {
         GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-        object_class->constructor = gnome_xsettings_manager_constructor;
         object_class->finalize = gnome_xsettings_manager_finalize;
 
         g_type_class_add_private (klass, sizeof (GnomeXSettingsManagerPrivate));
